<?php

namespace Controller;

use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\Mongo\Mongo;
use Util\Environment;

class ImportEvents {

    protected $dbNamePrefix = 'Zmart_';

    /** @var \Util\Environment $goalNotificationsService */
    protected $environment;
    /** @var \Service\GoalNotifications $goalNotificationsService */
    protected $goalNotificationsService;
    /** @var \Service\ImportHistory $importHistoryService */
    protected $importHistoryService;
    /** @var \Service\ManualImportQueueService $manualImportQueueService */
    protected $manualImportQueueService;
    /** @var \Service\ImportSchedule $importScheduleService */
    protected $importScheduleService;

    public function __construct(Environment $environment, $goalNotificationsService,
                                $importHistoryService, $manualImportQueueService,
                                $importScheduleService) {
        $this->environment = $environment;
        $this->goalNotificationsService = $goalNotificationsService;
        $this->importHistoryService = $importHistoryService;
        $this->manualImportQueueService = $manualImportQueueService;
        $this->importScheduleService = $importScheduleService;
    }

    public function finishImportation(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $organizationId = $row['organizationId'];
        $this->environment->setOrganizationId($organizationId);
        $this->checkOnManualImportQueue($row);
        if($row['importSucceeded']) {
            $this->checkGoalNotifications($row);
        }
    }

    protected function checkGoalNotifications($row) {
        $dataSourceId = $row['dataSourceId'];
        $this->goalNotificationsService->checkGoalNotifications($dataSourceId);
    }

    protected function checkOnManualImportQueue($row) {
        $processId = $row['processId'];
        $dataSourceId = $row['dataSourceId'];
        $process = $this->importHistoryService->findByProcessId($processId);
        if(!empty($process)) {
            $process = $process[0];
            if(isset($process['importType']) && $process['importType'] === 'manualBI') {
                $newSchedule = $this->manualImportQueueService->getItemToSchedule($dataSourceId);

                if($newSchedule) {
                    $this->manualImportQueueService->removeItemFromQueue($newSchedule);

                    unset($newSchedule['_id']);
                    $this->importScheduleService->saveOne($newSchedule);

                    $this->manualImportQueueService->updateProcessesOrders($dataSourceId);
                }
            }
        }
    }
}