<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Column extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_COLUMN = "579106a111e8f41bb00caff6";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $maskFunction = !empty($widget["chartSeries"][0]["serieYAxisFormat"]) ? $widget["chartSeries"][0]["serieYAxisFormat"] : null;
        $xAxis = [
            "type" => "value",
            "axisLabel" => [
                "show" => true,
            ],
        ];

        if (!empty($maskFunction)){
            $xAxis["maskFunction"] = $maskFunction;
        }
        return $xAxis;
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            if(!empty($row[$widget['chartXAxisField']]) && !in_array($row[$widget['chartXAxisField']], $categoria)){
                $categoria[] = $row[$widget['chartXAxisField']];
            }
        }
        $rotate = -60;
        if(!empty($widget['chartXAxisTicksRotation']) || (isset($widget['chartXAxisTicksRotation']) && $widget['chartXAxisTicksRotation'] === 0) )
        {
            $rotate = $widget['chartXAxisTicksRotation'];
        }
        $yAxis = [
            "type" => "category",
            "data" => $categoria,
            "axisLabel"=> [
                "rotate" =>  $rotate,
                "interval"=>  0,
                "overflow"=> "truncate",
                "hideOverlap"=> true
            ]
        ];
        if($rotate !== 0) {
            $yAxis["axisLabel"]["width"] = 100;
        }
        return $yAxis;
    }

    protected function getSeries()
    {
        $widget = $this->widget;
        $colors = $this->getSerieColors();
        $series = [];
        foreach ($this->widgetData as $row) {
            foreach ($widget['serieFields'] as $index => $fact) {
                if (empty($series[$index])) {
                    $series[$index] = array(
                        "name" => $widget['serieLabels'][$index],
                        "type" => "bar",
                        "data" => array(),
                        "label" => [
                            "show" => true,
                            "color"=> $colors[$index],
                            "position" => 'top'
                        ],
                        "stack" => $this->getSerieStacked(),
                    );
                }
                $series[$index]["data"][] = !empty($row[$fact]) ? $row[$fact] : 0;
            }
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $barOptions = [
            "horizontal" => true,
            "dataLabels" => [
                "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top',
                "textAnchor" => "end",
            ],
        ];

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barOptions["barHeight"] = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return [
            "bar" => $barOptions
        ];
    }

    protected function getBarWidth(){
        $widget = $this->widget;
        $barWidth = "auto";

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barWidth = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return $barWidth;
    }

    protected function getBarHeightByDimensionGap($dimensionGap){
        return strval((1-$dimensionGap)*100 )."%";
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "bar",
            "id" => $widget['id'],
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return array_merge($widget["chartSeries"][0]["serieColors"],self::DEFAULT_COLORS);
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["serieShowValueLabels"]) ? $widget["serieShowValueLabels"] : false,
            "position" => "top",
            "textAnchor" => "start",
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "textStyle" => [
                "fontSize" => "16px",
                "fontFamily" => "Arial Black",
                "fontWeight" => "bold",
            ],
            "align" => "left",
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
                "fontSize" => '14px',
            ],
        ];
    }

    protected function getLegend()
    {
        $widget = $this->widget;
        $string = $widget["chartLegendPosition"];
        $variable = [];
        $variations = explode(" ", $string);
        foreach ($variations as $variation) {
            if ($variation == "top") {
                $variable['top'] = "top";
                $variable['orient'] = "vertical";
            }
            if ($variation == "bottom") {
                $variable['bottom'] = "0%";
                $variable['orient'] = "vertical";
            }
            if ($variation == "left") {
                $variable['left'] = "left";
                $variable['orient'] = "vertical";
            }
            if ($variation == "right") {
                $variable['right'] = "right";
                $variable['orient'] = "vertical";
            }
        }
        return $variable;
    }

    protected function getDataZoom()
    {
        $data = $this->widgetData;
        $widget = $this->widget;
        $column =  count($data) * count($widget['serieFields']);
        $dataZoom  = null;
        if($column >= 20) {
            $end = intval(2000/$column);
            if($end < 1) {
                $end = 1;
            } else if($end > 100) {
                $end = 100;
            }
            $dataZoom = [
                [
                    "show" => true,
                    "height" => 10,
                    "end" => $end,
                    "moveHandleSize" => 0,
                    "bottom" => "0%"
                ]
            ];
        }
        return $dataZoom;
    }

    protected function getOptions(){
        $widget = $this->widget;

        $options = [
            "grid" => [
                "left" => '10px',
                "right" => '10px',
                "bottom" => '20px',
                "containLabel" => true
            ],
            "color" => $this->getSerieColors(),
            "legend" => $this->getLegend(),
            "title" => $this->getTitle(),
            "tooltip" => [
                "trigger" => "axis",
                "axisPointer" => [
                    "type" => "shadow"
                ]
            ],
            "xAxis" => $this->getXAxis(),
            "yAxis" => $this->getYAxis(),
            'series' => $this->getSeries(),
            "dataZoom" => $this->getDataZoom()
        ];

        if(empty($options["dataZoom"])){
            $options["grid"]["bottom"] = '0px';
        }
        return $options;
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_COLUMN,
            "component" => self::ECHART_COMPONENT,
            "chartType" => $widget['serieType'],
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
