<?php
namespace tests\Zeedhi\Report;

use Zeedhi\Report\LogoConfig;
use Zeedhi\Report\ReportBuilder;

class ReportBuilderTest extends \PHPUnit_Framework_TestCase {

    public function testBuildReport() {
        $dataSet = array(
            array("COUNTRY_ID" => "FR", "COUNTRY_NAME" => "France", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "UK", "COUNTRY_NAME" => "United Kingdom", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "NL", "COUNTRY_NAME" => "Netherlands", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "IT", "COUNTRY_NAME" => "Italy", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "DK", "COUNTRY_NAME" => "Denmark", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "DE", "COUNTRY_NAME" => "Germany", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "CH", "COUNTRY_NAME" => "Switzerland", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "BE", "COUNTRY_NAME" => "Belgium", "REGION_NAME" => "Europa"),
            array("COUNTRY_ID" => "AR", "COUNTRY_NAME" => "Argentina", "REGION_NAME" => "America"),
            array("COUNTRY_ID" => "MX", "COUNTRY_NAME" => "Mexico", "REGION_NAME" => "America"),
            array("COUNTRY_ID" => "CA", "COUNTRY_NAME" => "Canada", "REGION_NAME" => "America"),
            array("COUNTRY_ID" => "US", "COUNTRY_NAME" => "United States of America", "REGION_NAME" => "America"),
            array("COUNTRY_ID" => "BR", "COUNTRY_NAME" => "Brasil", "REGION_NAME" => "America"),
            array("COUNTRY_ID" => "CN", "COUNTRY_NAME" => "China", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "HK", "COUNTRY_NAME" => "HongKong", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "SG", "COUNTRY_NAME" => "Singapore", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "IN", "COUNTRY_NAME" => "India", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "JP", "COUNTRY_NAME" => "Japan", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "AU", "COUNTRY_NAME" => "Australia", "REGION_NAME" => "Asia"),
            array("COUNTRY_ID" => "KW", "COUNTRY_NAME" => "Kuwait", "REGION_NAME" => "Middle East and Africa"),
            array("COUNTRY_ID" => "IL", "COUNTRY_NAME" => "Israel", "REGION_NAME" => "Middle East and Africa"),
            array("COUNTRY_ID" => "EG", "COUNTRY_NAME" => "Egypt", "REGION_NAME" => "Middle East and Africa"),
            array("COUNTRY_ID" => "NG", "COUNTRY_NAME" => "Nigeria", "REGION_NAME" => "Middle East and Africa"),
            array("COUNTRY_ID" => "AT", "COUNTRY_NAME" => "Austrália", "REGION_NAME" => "Oceania")
        );

        $reportMetaData = array(
            "name"  => "countries_by_regions",
            "title" => "Contries by Regions",
            "version" => "1.0a",
            "columns" => array(
                "COUNTRY_ID" => array(
                    "descricao" => "Id",
                    "size" => "20%",
                    "align" => "left",
                    "format" => null,
                ),
                "COUNTRY_NAME" => array(
                    "descricao" => "Country Name",
                    "size" => "50%",
                    "align" => "left",
                    "format" => null,
                ),
                "REGION_NAME" => array(
                    "descricao" => "Region Name",
                    "size" => "30%",
                    "align" => "left",
                    "format" => null,
                ),
            ),
            "groups" => array(
                array(
                    "name"   => "group_by_region",
                    "field"  => "REGION_NAME",
                    "header" => array(
                        "label"        => "Region",
                        "displayfield" => "REGION_NAME",
                    ),
                    "footer" => array(
                        "label"      => "Countries in region",
                        "expression" => "COUNT(*)"
                    )
                )
            ),
            "reportSummary" => array(
                "label"      => "Total of countries",
                "expression" => "COUNT(*)"
            ),
            'clientLogo' => 'http://localhost/static/images/logo/logo-cliente.gif',
            'enterpriseLogo' => 'http://localhost/static/images/zeedhi-logo-min.png',
            'productLogo' => 'http://localhost/static/images/zeedhi-logo.png'
        );


        $logoConfig = new LogoConfig();
        $logoConfig->setClientLogo('http://localhost/images/clientLogo.png');
        $logoConfig->setEnterpriseLogo('http://localhost/images/enterpriseLogo.png');
        $logoConfig->setProductLogo('http://localhost/images/productLogo.png');

        $reportName = uniqid("report") . ".html";
        $targetFile = realpath(__DIR__) . "/" . $reportName;


        $builder = new ReportBuilder(
            $reportMetaData,
            $dataSet,
            $logoConfig,
            realpath(__DIR__."/../../../templates"),
            "seems unused", //@todo remove this param.
            $targetFile,
            'http://localhost/static/assets'
        );
        $builder->buildReport();

        $this->assertTrue(file_exists($targetFile));
        $this->assertTrue(unlink($targetFile));
    }

}